# import statements
#

class G2_memoryBank:
	'''instances of this class implement four memory banks, each with 8 words of 2 bytes.
	Each byte is a list of bits.'''
	
	def __init__(self):
		self.memoryBank=[[],[],[],[]]
		for mBank in range(0,4):
			self.memoryBank[mBank]=[[],[],[],[],[],[],[],[]]
			for mWord in range(0,8):
				self.memoryBank[mBank][mWord]=[[],[]]
				for mByte in range(0,2):
					self.memoryBank[mBank][mWord][mByte]=[0,0,0,0,0,0,0,0]
		#print 'memory bank 0 is',self.memoryBank[0]
		#print ' '
		#print ' '
		self.lockKill=0
		self.permalockKill=0
		self.lockAccess=0
		self.permalockAccess=0
		self.lockBank=['NULL',0,0,0]
		self.permalockBank=['NULL',0,0,0]
		self.banksize=8	#  maximum size of memory banks in words... for the moment same for all banks
	
	#  end instance initialization
	
	#  now define routines to convert between strings and numerical values
	
	def readByte(self,mBank,mWord,mByte):
		'''This is a private function to be called by listToByte.'''
		return self.memoryBank[mBank][mWord][mByte]
		
	def writeByte(self,mBank,mWord,mByte,byteList):
		'''This is a private function to be called by byteToList.'''
		self.memoryBank[mBank][mWord][mByte]=byteList
		
	#
	def readMemoryByte(self,mBank,mWord,mByte):
		''' This function converts a byte-list of bits into an integer byte and returns it.
		If the arguments are out of range it raises a ValueError exception.'''
		#
		if ((mBank<0) or (mBank>3)):
			raise ValueError, "Memory argument out of range"
			return 'NULL'
		elif ((mWord<0) or (mWord>7)):
			raise ValueError, "Memory argument out of range"
			return 'NULL'
		elif ((mByte<0) or (mByte>1)):
			raise ValueError, "Memory argument out of range"
			return 'NULL'
		else:
			byteValue=0
			templist=self.readByte(mBank,mWord,mByte)
			for i in range(0,8):
				byteValue=byteValue*2+templist[i]
			return byteValue
	#  end listToByte
	
	
	def writeMemoryByte(self,mBank,mWord,mByte,byteValue):
		'''This function converts an integer to a bit list and writes it
		to memory in the appropriate bank and word.  If the integer is 
		greater than 255, or the memory bank arguments are out of range, 
		a ValueError exception is raised.'''
		
		if ((mBank<0) or (mBank>3)):
			raise ValueError, "Memory argument out of range"
			return 'NULL'
		elif ((mWord<0) or (mWord>(self.banksize-1))):
			raise ValueError, "Memory argument out of range"
			return 'NULL'
		elif ((mByte<0) or (mByte>1)):
			raise ValueError, "Memory argument out of range"
			return 'NULL'
		elif byteValue>255:
			raise ValueError, "more than 8 bits in memory byte"
			return 'NULL'
		else:		
			#  do the conversion and write the byte to memory
			remainder=byteValue
			#print 'remainder is:',remainder
			binaryTuple=(128,64,32,16,8,4,2,1)
			byteList=[]
			for i in range(0,8):
				if remainder>(binaryTuple[i]-1):
					byteList=byteList+[1]
					remainder=remainder-binaryTuple[i]
					#print 'remainder is:', remainder
				else:
					byteList=byteList+[0]
					# remainder is unchanged	
			#print 'byteList in byteToList is', byteList
			self.writeByte(mBank,mWord,mByte,byteList)							
			#print 'memory bank 0 in byteToList is',self.memoryBank[0]
			return 'WRITE OK'
			
	#  end byteToList	
	
	
	def memoryToBitSequence(self,mBank):
		'''This function copies a whole memory bank to an unstructured list of bits and returns the list.'''
		memoryBits=[]
		for mWord in range(0,8):
			for mByte in range(0,2):
				memoryBits=memoryBits+self.memoryBank[mBank][mWord][mByte]
		return memoryBits
	#   end  memoryToBitSequence
	
	
	def matchMemory(self,mBank,Pointer,Length,Mask):
		'''This function compares a bit list Mask to the memory bit sequence starting at Pointer for Length.'''
		if (Length==len(Mask)):
			bankBits=self.memoryToBitSequence(mBank)	#    get the list of bits in the memory bank
			match=True
			for i in range(0,Length):
				if bankBits[Pointer+i]!=Mask[i]:
					match=False
			return match
		else: 
			raise ValueError, "Length doesn't match Mask"
	#  end matchMemory

#  G2_memoryBank   release 1.0
#  Daniel M. Dobkin   July 1 2006
#  Enigmatics
#  www.enigmatic-consulting.com



			
		
		

		